import requests
import validators

from pyModules.GuardPoint.api_cardholders import REQUEST_TIMEOUT
from pyModules.GuardPoint.api_exceptions import SecurityGroupsAPIError, UnauthorizedError
from pyModules.bearerToken import BearerAuth


class SecurityGroupsAPI:

    def __init__(self, app_config, token):
        if not validators.url(app_config['GP10']['base_url']):
            raise ValueError('Bad URL')
        self.url = app_config['GP10']['base_url'] + "/odata/api_SecurityGroups?$select=uid, name&$filter=name ne 'Anytime Anywhere'"
        self.auth = BearerAuth(token)

    def get_security_groups(self):

        response = requests.get(self.url, auth=self.auth, timeout=REQUEST_TIMEOUT)

        if response.status_code == requests.codes.ok:
            security_groups = response.json()
            security_groups_value = security_groups['value']
            return security_groups_value
        elif response.status_code == requests.codes.unauthorized:
            raise UnauthorizedError(f"Token Rejected from GP Server. {response.status_code}")
        else:
            raise SecurityGroupsAPIError(f"Failed to get back Security Groups. (HTTP Status Code: {response.status_code})")
