import uuid
import boto3
from datetime import datetime

dynamodb = boto3.resource('dynamodb')


class RegisterDataBaseDynamo(object):
    table = dynamodb.Table("visitor_requests")

    def generate_request_uid(self):
        # DynamoDB attribute of type Number can store 126-bit integers
        # Shift right to ensure number can be represented with 64 bits
        uid = str(uuid.uuid1().int >> 64)
        if len(uid) < 20:
            uid = uid.rjust(20, "1")
        if len(uid) > 20:
            uid = uid[0:20]

        return int(uid)

    def insert_visitor_request(self, firstName, lastName, company, email, photo, signature):
        assert isinstance(firstName, str) is True, "Visitor Requests must have a first name"
        assert isinstance(lastName, str) is True, "Visitor Requests must have a last name"

        uid = self.generate_request_uid()
        timestamp = int(datetime.now().timestamp())

        response = self.table.put_item(
            Item={
                'uid': uid,
                'timestamp': timestamp,
                'firstName': firstName,
                'lastName': lastName,
                'company': company,
                'email': email,
                'photo': photo,
                'signature': signature
            },
            ReturnValues='NONE'
        )

        assert int(response['ResponseMetadata']['HTTPStatusCode']) == 200, "New visitor request failed"

        return str(uid) + "-" + str(timestamp)

    def get_visitor_request(self, request_id):
        rid = request_id.split("-")

        response = self.table.get_item(
            Key={
                    'uid': int(rid[0]),
                    'timestamp': int(rid[1])
                },
            ConsistentRead=True
        )

        assert 'Item' in response, "Get visitor request failed (Missing Item)"
        assert int(response['Item']['uid']) == int(rid[0]), "Get visitor request failed (UID mismatch)"

        return {
                    "uid": response['Item']['uid'],
                    "timestamp": response['Item']['timestamp'],
                    "firstName": response['Item']['firstName'],
                    "lastName": response['Item']['lastName'],
                    "company": response['Item']['company'],
                    "email": response['Item']['email'],
                    "photo": response['Item']['photo']
                }

    def get_visitor_requests(self, offset=0, count=10):

        response = self.table.scan()

        return response['Items']
