from datetime import datetime
from enum import Enum

import jwt as jwt

from pyModules.GuardPoint.api_exceptions import UnauthorizedError
from pyModules.GuardPoint.api_token import TokenAPI

class AuthToken:
    secret = ""

    def __init__(self, app_config):
        self.api_token = TokenAPI(app_config)
        self.token_pwd = app_config['GP10']['token_pwd']
        self.guest_ops = app_config['Operations']['guest'].split(',')
        self.admin_ops = app_config['Operations']['admin'].split(',')
        self.guest_user = app_config['GP10']['guest_user']
        self.guest_pwd = app_config['GP10']['guest_pwd']

    def kdf(self):
        ''' IP Address, Browser headers - add something to link browser to token'''
        pass

    def create(self, user_name, password):
        if user_name == 'guest':
            gp_token = self.api_token.get_token(self.guest_user, self.guest_pwd)
            token_type = 'guest'
        else:
            gp_token = self.api_token.get_token(user_name, password)
            token_type = 'admin'

        timestamp = int(datetime.now().timestamp())

        token = jwt.encode({"gpt": gp_token, "ts": timestamp, "type": token_type}, self.token_pwd,
                           algorithm="HS256", headers={"ts": timestamp, "type": token_type})

        return token

    # Returns the GP Token
    def verify(self, token, operation):
        decoded = jwt.decode(token, self.token_pwd, algorithms="HS256")
        header = jwt.get_unverified_header(token)

        gp_token = decoded['gpt']
        token_ts = int(decoded['ts'])
        token_type = decoded['type']

        # Check hdr time matches enc payload time
        if token_type != header['type']:
            raise UnauthorizedError("Corrupt Token Type")
        if token_ts != int(header['ts']):
            raise UnauthorizedError("Corrupt Token Timestamp")

        if token_type == 'admin':
            if (operation not in self.admin_ops) and (operation not in self.guest_ops):
                raise UnauthorizedError("Operation Not Permitted")
        else:
            if operation not in self.guest_ops:
                raise UnauthorizedError("Operation Not Permitted")

        # Check timestamp age
        timestamp = int(datetime.now().timestamp())
        diff = timestamp - token_ts
        if diff > (60 * 59):  # 59 mins
            raise UnauthorizedError("Token Expired")

        return gp_token, token_type

    def renew(self, token, token_type):

        gp_token = self.api_token.renew_token(token)
        timestamp = int(datetime.now().timestamp())

        token = jwt.encode({"gpt": gp_token, "ts": timestamp, "type": token_type}, self.token_pwd,
                           algorithm="HS256", headers={"ts": timestamp, "type": token_type})

        return token
