import validators
from pyModules.RequestDatabase.register_db import RegisterDataBase
from pyModules.utils import is_base64
from pyModules.Email.sEmail import VisitorEmail

MAX_PHOTO_SIZE = 900000


def handle_get_visitor_requests(app_config):
    db = RegisterDataBase(app_config)
    requests = db.get_visitor_requests()
    return requests


def handle_visitor_lookup(app_config, rid):
    if len(rid) < 10:
        raise ValueError('Incorrect rid length')
    if '-' not in rid:
        raise ValueError('Malformed rid')

    db = RegisterDataBase(app_config)
    visitor_entry = db.get_visitor_request(str(rid))
    return visitor_entry


def handle_new_visitor_request(params, app_config):
    # Perform User Parameter Validation

    # 1. Check Mandatory Parameters exist
    if "firstName" not in params:
        raise ValueError('Missing firstName')
    if "lastName" not in params:
        raise ValueError('Missing lastName')
    if "company" not in params:
        raise ValueError('Missing company')
    if "email" not in params:
        raise ValueError('Missing Email')
    if "fromDateTimeValid" not in params:
        raise ValueError('Missing fromDateTimeValid')
    if "toDateTimeValid" not in params:
        raise ValueError('Missing toDateTimeValid')

    # 2. Nullify Optional Parameters
    if "photo" not in params:
        params['photo'] = None
    if "signature" not in params:
        params['signature'] = None

    # 2. Check Lengths
    if len(params['firstName']) < 1 or len(params['firstName']) > 100:
        raise ValueError('firstName length')
    if len(params['lastName']) < 1 or len(params['lastName']) > 100:
        raise ValueError('lastName length')
    if len(params['email']) < 3:
        raise ValueError('email length')

    if params['photo']:
        if len(params['photo']) < 100:
            raise ValueError('photo length')
        if len(params['photo']) > MAX_PHOTO_SIZE:
            raise ValueError('Photo too big')

    if params['signature']:
        if len(params['signature']) < 100:
            raise ValueError('Signature length')
        if len(params['signature']) > MAX_PHOTO_SIZE:
            raise ValueError('Signature too big')

    # 3. Type Checks
    if type(params['firstName']) != str:
        raise TypeError("firstName must be a string")
    if type(params['lastName']) != str:
        raise TypeError("lastName must be a string")
    if type(params['company']) != str:
        raise TypeError("company must be a string")

    if params['photo']:
        if not is_base64(params['photo']):
            raise ValueError('Photo encoding')

    if params['signature']:
        if not is_base64(params['signature']):
            raise ValueError('Signature encoding')

    # 4. Syntax Checks
    if not validators.email(params['email']):
        raise ValueError(f'Invalid Email')

    db = RegisterDataBase(app_config)
    rid = db.insert_visitor_request(params['firstName'], params['lastName'], params['company'], params['email'],
                                    params['photo'], params['signature'])

    new_email = VisitorEmail(app_config,
                             name=f"{params['firstName']} {params['lastName']}",
                             company=params['company'],
                             rid=rid,
                             photo=params['photo'],
                             signature=params['signature']
                             )
    new_email.send()

    return rid
