var utils = {
    /* Every invocation resets the timer, and reschedules the func 300 ms in the future. */
    debounce: function(func, timeout = 300) {
      let timer;
      return (...args) => {
        clearTimeout(timer);
        timer = setTimeout(() => { func.apply(this, args); }, timeout);
      };
    },

    /* Separates first and last name */
    splitNames: function(nameSearchInput) {
         firstName = '';
         lastName = '';
         if(nameSearchInput.length > 3) {
            var res = nameSearchInput.split(" ", 3);
            firstName = res[0];
            if(res.length > 1) {
                lastName = res[1];
            }
         }
         return [firstName, lastName];
    }

};
var photo = {
    file_elem: null,
    preview_elem: null,
    spinner_elem: null,
    reader: null,
    unscaled_image: null,
    MAX_PHOTO_WIDTH: 350,
    MAX_PHOTO_HEIGHT: 450,
    custom_photo: false,


    setup: function(file_elem, preview_elem, spinner_elem){
        if(file_elem instanceof HTMLInputElement && file_elem.getAttribute('type') == 'file'){
            photo.file_elem = file_elem;
        } else {
            console.error("incorrect photo element type: " + input_elem.getAttribute('type'));
        }

        if(preview_elem instanceof HTMLImageElement){
            photo.preview_elem = preview_elem;
        } else {
            console.error("Photo preview element is not a IMG.");
        }

        if(spinner_elem instanceof HTMLDivElement){
            photo.spinner_elem = spinner_elem;
        } else {
            console.error("Spinner element is not a DIV.");
        }

        /* Logs when custom photo is successfully uploaded */
        photo.preview_elem.addEventListener("load", function () {
            console.log('photo.preview_elem size: ' + photo.preview_elem.width + 'x' + photo.preview_elem.height);
            photo.custom_photo = true;
        }, false);

        /* Runs a spinner only */
        photo.file_elem.onclick = function() {
            console.log("photo onclick");
            // Run the Spinner for 5 secs
            photo.spinner_elem.style.visibility = "visible";
            setTimeout(function(){ photo.spinner_elem.visibility = "hidden"; }, 5000);
        }

        /* Checks file is a Image and is not too big only */
        photo.file_elem.onchange  = function() { photo.onchange(); }

        /* Image is loaded into 'unscale_image' first, once loaded the image is resized and placed into preview_elem */
        photo.unscaled_image = new Image();
        photo.unscaled_image.addEventListener("load", function () { photo.resize(); }, false);

        photo.reader = new FileReader();
        photo.reader.addEventListener("load", function () {
            console.log("image loaded into unscaled_image");
            // convert image file to base64 string
            photo.unscaled_image.src = photo.reader.result;
            // Stop the Spinner
            photo.spinner_elem.style.visibility = "hidden";
        }, false);
    },

    /* Checks file is a Image and is not too big */
    onchange: function(){
        console.log("photo onchange");
        if(photo.file_elem.files[0]){
            if(!photo.file_elem.files[0].type.match(/image.*/)) {
                alert("File is not a image");
                return;
            }

            if(photo.file_elem.files[0].size > 10000000) {
                alert("Photo too big! (10mb Max)");
                photo.file_elem.value = "";
                photo.file_elem.files[0] = null;
                photo.preview_elem.src = "data:image/png;charset=utf-8;base64," + default_photo_src;
                photo.spinner_elem.style.visibility = "hidden";
                return;
            }

            /* Load the image into the reader */
            photo.reader.readAsDataURL(photo.file_elem.files[0]);

        } else {
            photo.spinner_elem.style.visibility = "hidden";
            photo.file_elem.files[0] = null;
        }
    },

    resize: function() {
        const canvas = document.createElement('canvas');
        console.log('photo.unscaled_image Image has changed');
        console.log('photo.unscaled_image size: ' + photo.unscaled_image.width + 'x' + photo.unscaled_image.height);
        if(photo.unscaled_image.width > photo.MAX_PHOTO_WIDTH) {
            ratio = (photo.unscaled_image.width/photo.MAX_PHOTO_WIDTH);
            new_height = photo.unscaled_image.height / ratio;
            new_width = photo.unscaled_image.width / ratio;

            if(new_height > photo.MAX_PHOTO_HEIGHT) {
                ratio = (photo.unscaled_image.height/photo.MAX_PHOTO_HEIGHT);
                new_height = photo.unscaled_image.height / ratio;
                new_width = photo.unscaled_image.width / ratio;
            }

            canvas.width = new_width;
            canvas.height = new_height;
            canvas.getContext('2d').drawImage(photo.unscaled_image, 0, 0, photo.unscaled_image.width, photo.unscaled_image.height, 0, 0, new_width, new_height);
            photo.preview_elem.src = canvas.toDataURL('image/jpeg', 1.0);
        } else {
            photo.preview_elem.src = photo.unscaled_image.src;
        }
    }

};
var apiVmsRegister = {

    api_url: window.location.protocol + "//" + window.location.host + window.location.pathname.replace(/\/+$/, '').replace('/register', '') +"/api",

    newVisitorRequest: function(firstName, lastName, company, email, photoB64, fromDateTimeValid, toDateTimeValid, signatureB64,
                                onSuccessFunc, onFailureFunc){
        console.log('Submitting newVisitorRequest ... ');
        const data = { operation: 'newVisitorRequest', firstName: firstName, lastName: lastName, company: company,
         email: email, photo: photoB64, fromDateTimeValid: fromDateTimeValid, toDateTimeValid: toDateTimeValid,
         signature: signatureB64 };
        console.log('api_url: ' + apiVmsRegister.api_url);
        apiVmsRegister.post(apiVmsRegister.api_url, data, onSuccessFunc, onFailureFunc);
    },

    post: function(url, data, onSuccessFunc, onFailureFunc) {
        let httpStatusCode = null;
        fetch(url, {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify(data),
        })
        .then( response => {
            httpStatusCode = response.status;
            return response.text();
        })
        .then( bodyAsText => {
            try {
                const bodyAsJson = JSON.parse(bodyAsText);
                return bodyAsJson;
            } catch (e) {
                onFailureFunc("JSON.parse Failure");
            }
        })
        .then(bodyAsJson => {
            /*console.log('HTTP Status Code: ', httpStatusCode);
            console.log('bodyAsJson: ', bodyAsJson);*/
            if(httpStatusCode == 200) {
                /* Got OK response(200) */
                if(bodyAsJson !== null) {
                    if(bodyAsJson.hasOwnProperty('errorMessage')){
                        console.error('Unclassified Error! ', bodyAsJson.errorMessage);
                        onFailureFunc(bodyAsJson.errorMessage);
                    } else {
                        onSuccessFunc(bodyAsJson);
                    }
                } else {
                    onSuccessFunc(bodyAsJson);
                }
            } else {
                /* Got Error response(200) */
                if(bodyAsJson !== null) {
                    if(bodyAsJson.hasOwnProperty('errorMessage')){
                        onFailureFunc(bodyAsJson.errorMessage);
                    } else {
                        onFailureFunc(str(httpStatusCode) + "- No errorMessage");
                    }
                } else {
                    onFailureFunc(str(httpStatusCode) + "- No errorMessage");
                }
            }
        })
        .catch((error) => {
          console.log('HTTP Status Code: ', httpStatusCode);
          console.error('Error:', error);
          console.error('Error message:', error.errorMessage);
          onFailureFunc(error);
        });
    }
};
var sigCanvas = {
    canvas: document.createElement('canvas'),
    ctx: null,
    pos: { x: 0, y: 0 },
    offset: { left: 0, top: 0 },
    scroll: { x: 0, y: 0 },
    element: null,

    init: function(element){
        sigCanvas.element = element;
        sigCanvas.ctx = sigCanvas.canvas.getContext('2d');
        rect = element.getBoundingClientRect();
        sigCanvas.scroll.x = window.scrollX;
        sigCanvas.scroll.y = window.scrollY;
        sigCanvas.offset.left = rect.left;
        sigCanvas.offset.top = rect.top;
        sigCanvas.canvas.width = rect.width;
        sigCanvas.canvas.height = rect.height;
        element.appendChild(sigCanvas.canvas);
    },

    getSnapShot: function(e) {
        if (!sigCanvas.canvas) {
            console.error("sigCanvas must be initialised with init");
            return
        } else {
            return sigCanvas.canvas.toDataURL();
        }
    },

    setTouchPos: function(e) {
        console.log("setTouchPos");
        if (!e)
            var e = event;

        if(e.touches) {
            if (e.touches.length == 1) { // Only deal with one finger
                var touch = e.touches[0]; // Get the information for finger #1
                sigCanvas.pos.x=touch.pageX-touch.target.offsetLeft;
                sigCanvas.pos.y=touch.pageY-touch.target.offsetTop;
            }
        }
        e.preventDefault();
        //alert("sigCanvas.pos.y:" + sigCanvas.pos.y);
    },

    setMousePos: function(e) {
        console.log("setMousePos");
        if((sigCanvas.scroll.x != window.scrollX) || (sigCanvas.scroll.y != window.scrollY)){
            sigCanvas.scroll.x = window.scrollX;
            sigCanvas.scroll.y = window.scrollY;
            rect = sigCanvas.element.getBoundingClientRect();
            sigCanvas.offset.left = rect.left;
            sigCanvas.offset.top = rect.top;
            /*console.log("e.pageY:" + e.pageY);
            console.log("sigCanvas.offset.top:" + sigCanvas.offset.top);
            console.log("sigCanvas.scroll.y:" + sigCanvas.scroll.y);*/
        }
        sigCanvas.pos.x = e.pageX - (sigCanvas.offset.left + sigCanvas.scroll.x);
        sigCanvas.pos.y = e.pageY - (sigCanvas.offset.top + sigCanvas.scroll.y);
    },

    mouseDraw: function(e) {
        if (!sigCanvas.ctx) { console.error("sigCanvas must be initialised with init"); return}
        // mouse left button must be pressed
        if (e.buttons !== 1) return;

        sigCanvas.ctx.beginPath(); // begin

        sigCanvas.ctx.lineWidth = 5;
        sigCanvas.ctx.lineCap = 'round';
        sigCanvas.ctx.strokeStyle = '#c0392b';

        sigCanvas.ctx.moveTo(sigCanvas.pos.x, sigCanvas.pos.y); // from
        sigCanvas.setMousePos(e);
        sigCanvas.ctx.lineTo(sigCanvas.pos.x, sigCanvas.pos.y); // to

        sigCanvas.ctx.stroke(); // draw it!
    },

    touchDraw: function(e) {
        // Update the touch co-ordinates
        //sigCanvas.setTouchPos(e);

        sigCanvas.ctx.beginPath(); // begin

        sigCanvas.ctx.lineWidth = 5;
        sigCanvas.ctx.lineCap = 'round';
        sigCanvas.ctx.strokeStyle = '#c0392b';

        sigCanvas.ctx.moveTo(sigCanvas.pos.x, sigCanvas.pos.y); // from
        sigCanvas.setTouchPos(e);
        sigCanvas.ctx.lineTo(sigCanvas.pos.x, sigCanvas.pos.y); // to

        sigCanvas.ctx.stroke(); // draw it!

        // Prevent a scrolling action as a result of this touchmove triggering.
        e.preventDefault();
    }
};
window.onload = function(){

    // create canvas element and append it to document body
    let sig_box = document.getElementById("signature_box");
    sigCanvas.init(sig_box);
    sig_box.addEventListener('mousemove', function(e){ sigCanvas.mouseDraw(e); });
    sig_box.addEventListener('mousedown', function(e){ sigCanvas.setMousePos(e);});
    sig_box.addEventListener('mouseenter', function(e){ sigCanvas.setMousePos(e);});
    sig_box.addEventListener('touchmove', function(e){ sigCanvas.touchDraw(e); });
    sig_box.addEventListener('touchstart', function(e){ sigCanvas.setTouchPos(e);});
    //sig_box.addEventListener('touchend', function(e){ sigCanvas.setTouchPos(e);});

    //Set DateTime defaults
    let fromDateValid   = document.getElementById("fromDateValid");
    let fromTimeValid   = document.getElementById("fromTimeValid");
    let toDateValid     = document.getElementById("toDateValid");
    let toTimeValid     = document.getElementById("toTimeValid");

    fromDateValid.defaultValue = getTodaysDate()
    toDateValid.defaultValue = getTomorrowsDate()

    photo.setup(document.getElementById("photo_file"), document.getElementById("photo_preview"), document.getElementById("photo_spinner"));
};

const newVisitorSubmit = utils.debounce(apiVmsRegister.newVisitorRequest, timeout = 1000);

function newVisitorSubmitBtnClick(){
    let firstName       = document.getElementById("firstName").value;
    let lastName        = document.getElementById("lastName").value;
    let company         = document.getElementById("company").value;
    let email           = document.getElementById("email").value;
    let photoSrc        = document.getElementById("photo_preview").src;
    let fromDateValid   = document.getElementById("fromDateValid").value;
    let fromTimeValid   = document.getElementById("fromTimeValid").value;
    let toDateValid     = document.getElementById("toDateValid").value;
    let toTimeValid     = document.getElementById("toTimeValid").value;
    let timeZoneOffset  = document.getElementById("timezone_offset").value;

    let resultMsg       = document.getElementById("newVisitorSubmitResultMsg");
    let spinner         = document.getElementById("newVisitorSubmitSpinner");
    let signature       = sigCanvas.getSnapShot();

    resultMsg.innerHTML = "hello";

    var photoB64 = null;
    if (photo.custom_photo && photoSrc.match(/^data:image\/(png|jpg|jpeg);base64,/,)) {
        photoB64 = photoSrc.replace(/^data:image\/(png|jpg|jpeg);base64,/, "");
    }

    var signatureB64 = null;
    if (signature.match(/^data:image\/(png|jpg|jpeg);base64,/,)) {
        signatureB64 = signature.replace(/^data:image\/(png|jpg|jpeg);base64,/, "");
    }

    //Start the spinner
    spinner.style.visibility = "visible";
    resultMsg.innerHTML = "loading ...";

    // Convert Date&Time to UTC ISO Strings
    fromDateTimeValid   = new Date(fromDateValid + "T" + fromTimeValid + ":00" + timeZoneOffset);
    toDateTimeValid     = new Date(toDateValid + "T" + toTimeValid + ":00" + timeZoneOffset);

    newVisitorSubmit(firstName, lastName, company, email, photoB64, fromDateTimeValid, toDateTimeValid, signatureB64,
        function (obj) { /* OnSuccess */
            //alert("Success rid=" + obj.rid);
            resultMsg.innerHTML = "Success";
            document.open();
            document.write("<html><body><h1>Visitor request sent successfully.</h1><form action='/register'><input type='submit' value='Go Back' /></form></body></html>");
            document.close();
        },
        function (msg) { /* OnFailure */
           //alert(msg);
           resultMsg.innerHTML = msg;
        },);

}

function getTodaysDate() {
    var local = new Date();
    local.setMinutes(local.getMinutes() - local.getTimezoneOffset());
    return local.toJSON().slice(0,10);
};
function getTomorrowsDate() {
    var local = new Date();
    local.setDate(local.getDate() + 1);
    local.setMinutes(local.getMinutes() - local.getTimezoneOffset());
    return local.toJSON().slice(0,10);
};