import {apiVms} from "/js/modules/api_vms.mjs";
import {cookieVms} from "/js/modules/cookie.mjs";
import {utils} from "/js/modules/utils.mjs";
import {photo} from "/js/modules/photo.mjs";
import {cardHolderTable} from "/js/modules/table.mjs";
import {cardHolderForm} from "/js/modules/cardHolderForm.mjs";

// Run once page has loaded
window.onload = function(){

    /* Listen to Scroll Event */
    document.getElementById("content").addEventListener('scroll', () => {
        let content = document.getElementById("content");

        if(content.scrollTop >= ((content.scrollHeight-1) - content.offsetHeight))
        {
            contentScrollEvent();
        };

    });

    /* Listen to Name Search Return Key Press */
    document.getElementById("nameSearchInput").addEventListener('keydown', (e) => {
        if(e.code === 'Enter') {
            nameSearchSubmit()
        };
    });

    /* Check if rid is present in the querystring */
    let url = new URL(document.location)
    let urlSearchParams = url.searchParams;

    if(urlSearchParams.has("rid")) {
        cardHolderForm.displayNewCardHolderForm(urlSearchParams.get("rid"));
        urlSearchParams.delete('rid');
    } else {
        /* Table ID */
        if(urlSearchParams.has("tid")) {
            let table_id = urlSearchParams.get("tid");
            if(table_id == 1){
                /* Column Layout 1 */
                cardHolderTable.col_widths  = ['2%', '20%', '10%',    '10%',                   '10%',        '18%', '10%',     '10%',    '10%'],
                cardHolderTable.col_hdrs    = ['', 'Name', 'CardCode', 'Visitor Signature', 'Host Signature', 'Valid', 'Status', 'Group', 'Delete'];
            } else if(table_id == 2) {
                /* Column Layout 2 */
                cardHolderTable.col_widths  = ['2%', '25%', '10%',    '10%',                   '10%',        '18%', '10%',     '15%'],
                cardHolderTable.col_hdrs    = ['', 'Name', 'CardCode', 'Visitor Signature', 'Host Signature', 'Valid', 'Status', 'Group'];
            } else {
                /* Default Columns */
                cardHolderTable.col_widths  = ['2%', '10%',  '20%',   '10%',      '19%',    '9%',   '10%',    '10%',    '10%'],
                cardHolderTable.col_hdrs    = ['', 'Photo', 'Name', 'CardCode', 'Valid', 'Status', 'Group', 'Badge', 'Delete'];
            }


        }
        /* Display the card holder table */
        tableLoad();
    }
};

/* Document (DOM) Events (Debounce will prevent an event occurring twice within a set timeout)*/
const tableLoad = () => {
    cardHolderTable.refresh("loading ....", 10);
}
window.contentScrollEvent = utils.debounce(() => cardHolderTable.update(4), 500);
window.nameSearchSubmit = utils.debounce(() => cardHolderTable.refresh("Searching ...."));
window.sendEmailToCardHolderBtnClick = utils.debounce(sendEmail, 800);
window.deleteCardHolderBtnClick = utils.debounce(deleteCardHolder, 800);
window.displayNewCardHolderFormBtnClick = utils.debounce(cardHolderForm.displayNewCardHolderForm, 800);
window.createNewCardHolderBtnClick = utils.debounce(cardHolderForm.createNewCardHolder, 800);
window.logoutBtnClick = utils.debounce(logout, 800);
window.genBadgeBtnClick = utils.debounce(genCardHolderQR, 800); //genCardHolderQR(cardHolderUID, cardCode)
window.previewFileChange = previewFileChange;
window.previewFileClick = previewFileClick;

const MAX_PHOTO_WIDTH = 350;
const MAX_PHOTO_HEIGHT = 450;

function logout(){
    console.log("Sensor VMS logging out .....");
    cookieVms.erase();
    window.location.href = '/login.html' + window.location.search;
}

function sendEmail(cardHolderUID){
    let spinnerMsg = document.getElementById("emailSpinnerMsg");
    spinnerMsg.innerHTML = "...";
    let spinner = document.getElementById("emailSpinner");
    spinner.style.display = "block";

    apiVms.emailCardHolder(cardHolderUID,
        function () { /* OnSuccess */
           spinner.style.display = "none";
           spinnerMsg.innerHTML = "Email Sent";
        },
        function (msg) { /* OnFailure */
           spinner.style.display = "none";
           spinnerMsg.innerHTML = "Email failure: " + msg;
        },);
}

function deleteCardHolder(elem, cardHolderUID){
    let row = elem.parentNode.parentNode;
    row.style.backgroundColor = "red";
    apiVms.deleteCardHolder(cardHolderUID,
        function() {  /* OnSuccess */
             row.parentNode.removeChild(row);
        },
        function(msg) {  /* OnFailure */
             alert(msg);
             row.style.backgroundColor = "transparent";
        })
}

function previewFileClick() {
    console.log("Custom photo button press");
    let photoSpinner = document.getElementById('photoSpinner');
    photoSpinner.style.visibility = "visible";
    setTimeout(function(){ photoSpinner.style.visibility = "hidden"; }, 15000);
}

function previewFileChange() {
  console.log("Custom photo change");
  let preview = document.getElementById('photo_preview');
  let unscaled_image = new Image();
  let photo   = document.getElementById('photo');
  let photoSpinner = document.getElementById('photoSpinner');

  if(photo.files[0]){
      if(!photo.files[0].type.match(/image.*/)) {
        alert("File is not a image");
        return;
      }
      if(photo.files[0].size > 10000000){
        alert("Photo too big! (10mb Max)");
        photo.value = "";
        photo.files[0] = null;
        preview.src = "data:image/png;charset=utf-8;base64," + default_photo_src;
        photoSpinner.style.visibility = "hidden";
        return;
      }
  } else {
    photoSpinner.style.visibility = "hidden";
    photo.files[0] = null;
  }

  const reader = new FileReader();

  reader.addEventListener("load", function () {
    // convert image file to base64 string
    unscaled_image.src = reader.result;
    console.log("Custom photo updated");
    cardHolderForm.custom_photo = true;
    // Stop the Spinner
    photoSpinner.style.visibility = "hidden";
  }, false);

  unscaled_image.addEventListener("load", function () {
    const canvas = document.createElement('canvas');
    console.log('unscaled_image size: ' + unscaled_image.width + 'x' + unscaled_image.height);

    let ratio = (unscaled_image.width/MAX_PHOTO_WIDTH);
    let new_height = unscaled_image.height / ratio;
    let new_width = unscaled_image.width / ratio;

    if(new_height > MAX_PHOTO_HEIGHT) {
        ratio = (unscaled_image.height/MAX_PHOTO_HEIGHT);
        new_height = unscaled_image.height / ratio;
        new_width = unscaled_image.width / ratio;
    }

    canvas.width = new_width;
    canvas.height = new_height;
    canvas.getContext('2d').drawImage(unscaled_image, 0, 0, unscaled_image.width, unscaled_image.height, 0, 0, new_width, new_height);
    preview.src = canvas.toDataURL('image/jpeg', 1.0);
    console.log('preview.src len: ' + preview.src.length);
    console.log('scaled_image size: ' + new_width + 'x' + new_height);

  }, false);

  preview.addEventListener("load", function () {
    console.log('preview Image has changed');
  }, false);

  if (photo.files[0]) {
        reader.readAsDataURL(photo.files[0]);
  }
}

function genCardHolderQR(cardHolderUID, cardCode) {
    let modal = document.getElementById("myModal");
    let modalImg = document.getElementById("img01");
    let spinner = document.getElementById("imgSpinner");
    let captionText = document.getElementById("caption");
    let span = document.getElementsByClassName("close")[0];
    let emailDiv = document.getElementById("emailCardHolderDiv");
    span.onclick = function() {
      modal.style.display = "none";
    }

    /* Display the Modal */
    //modalImg.src = 'data:image/jpeg;charset=utf-8;base64,' + loading_badge_src;
    modal.style.display = "block";

    apiVms.getBadgePreview(cardHolderUID,
        function(data) {  /* OnSuccess */
            modalImg.src = 'data:image/png;charset=utf-8;base64,' + data.qr;
            captionText.innerHTML = data.cardCode;
            emailDiv.innerHTML  = '<button onclick="sendEmailToCardHolderBtnClick(' + "'"  + cardHolderUID + "'" + ')">Email CardHolder</button>';
        },
        function(msg) {  /* OnFailure */
            alert("Failed to get Badge Preview");
        });
}

