import {cookieVms} from "/js/modules/cookie.mjs"

export const apiVms = {

    url: window.location.protocol + "//" + window.location.host + "/api",

    getGuestToken: function(onSuccessFunc, onFailureFunc) {
        apiVms.getToken("guest", "visitor", onSuccessFunc, onFailureFunc);
    },

    getToken: function(username, password, onSuccessFunc, onFailureFunc) {
        console.log('Sending getToken request ... ');
        const data = { operation: 'getToken', username: username, password: password };
        apiVms.post(data, onSuccessFunc, onFailureFunc);
    },

    renewToken: function(onSuccessFunc, onFailureFunc) {
        console.log('Sending renewToken request ... ');
        const data = { token: cookieVms.get(), operation: 'renewToken'};
        apiVms.post(data,
            function(resp){
                cookieVms.set(resp.token);
                onSuccessFunc();
            },
            function(msg){
                onFailureFunc(msg);
            }
        );
    },

    checkToken: function(onSuccessFunc, onFailureFunc, is_guest = true){
        let token = cookieVms.get();
        if((token == null) && (is_guest)){
            apiVms.getGuestToken(
                function(resp){
                    cookieVms.set(resp.token);
                    onSuccessFunc();
                },
                function(msg){
                    onFailureFunc(msg);
                }
            )
        } else if(token == null) {
            location.href = "/login.html" + window.location.search;
        } else {
            let jwt = cookieVms.parseJwt(token);
            if(!is_guest){ // Admin
                if(jwt.type != "admin"){
                    location.href = "/login.html" + window.location.search;
                }
            }
            /* If token more than 30mins old - Renew  */
            let secs = Math.round(Date.now() / 1000);
            if(secs > cookieVms.parseJwt(token).ts + (30 * 60)){
                console.log("ts limit: " + (cookieVms.parseJwt(token).ts + (30 * 60)).toString());
                console.log("secs: " + secs.toString());
                apiVms.renewToken(onSuccessFunc, onFailureFunc);
            } else {
                onSuccessFunc();
            }
        }
    },

    getOffsiteCount: function (onSuccessFunc, onFailureFunc) {
        console.log('Sending getOffsiteCount request ... ');
        apiVms.checkToken(
            function(){
                let data = { token: cookieVms.get(), operation: 'getOffsiteCount' };
                apiVms.post(data, onSuccessFunc, onFailureFunc);
            },
            function(msg){
                onFailureFunc(msg);
            },
            true // is_guest
        );
    },

    getOnsiteCount: function (onSuccessFunc, onFailureFunc) {
        console.log('Sending getOnsiteCount request ... ');
        apiVms.checkToken(
            function(){
                let data = { token: cookieVms.get(), operation: 'getOnsiteCount' };
                apiVms.post(data, onSuccessFunc, onFailureFunc);
            },
            function(msg){
                onFailureFunc(msg);
            },
            true // is_guest
        );
    },

    getOnsiteCardholders: function (offset, count, onSuccessFunc, onFailureFunc) {
        console.log('Sending getOnsiteCardholders request ... ');
        apiVms.checkToken(
            function(){
                let data = { token: cookieVms.get(), operation: 'getOnsiteCardholders', offset: offset, count: count};
                apiVms.post(data, onSuccessFunc, onFailureFunc);
            },
            function(msg){
                onFailureFunc(msg);
            },
            true // is_guest
        );
    },

    getOffsiteCardholders: function (offset, count, onSuccessFunc, onFailureFunc) {
        console.log('Sending getOffsiteCardholders request ... ');
        apiVms.checkToken(
            function(){
                let data = { token: cookieVms.get(), operation: 'getOffsiteCardholders', offset: offset, count: count};
                apiVms.post(data, onSuccessFunc, onFailureFunc);
            },
            function(msg){
                onFailureFunc(msg);
            },
            true // is_guest
        );
    },

    emailSearchCardHolder: function (email, onSuccessFunc, onFailureFunc) {
        console.log('Sending emailSearchCardHolder request ... ');
        apiVms.checkToken(
            function(){
                let data = { token: cookieVms.get(), operation: 'emailSearchCardHolder', email: email };
                apiVms.post(data, onSuccessFunc, onFailureFunc);
            },
            function(msg){
                onFailureFunc(msg);
            },
            true
        );
    },

    updateCardHolderSignatures: function(uid, v_sig, h_sig, nda, onSuccessFunc, onFailureFunc){
        console.log('Sending updateCardHolderSignatures request ... ');
        apiVms.checkToken(
            function(){
                let data = { token: cookieVms.get(), operation: 'updateCardHolderSignatures', uid: uid,
                 visitorSignature: v_sig, hostSignature: h_sig, ndaAgreed: nda };
                apiVms.post(data, onSuccessFunc, onFailureFunc);
            },
            function(msg){
                onFailureFunc(msg);
            },
            true
        );
    },

    getVisitorRequest: function (requestID, onSuccessFunc, onFailureFunc) {
        console.log('Sending getVisitorRequest request ... ');
        const data = { token: cookieVms.get(), operation: 'getVisitorRequest', rid: requestID };
        apiVms.post(data, onSuccessFunc, onFailureFunc);
    },

    getBadgePreview: function (cardHolderUID, onSuccessFunc, onFailureFunc) {
        console.log('Sending getBadgePreview request ... ');
        const data = { token: cookieVms.get(), operation: 'genCardHolderQR', uid: cardHolderUID };
        apiVms.post(data, onSuccessFunc, onFailureFunc);
    },

    /* Returns the HTML Cardholder JSON object on success */
    getCardHolders: function (offset=0, limit=10, search_phrase="", onSuccessFunc, onFailureFunc) {
        console.log('Sending getCardHolders request ... ');
        apiVms.checkToken(
            function(){
                let data = { token: cookieVms.get(), operation: 'getCardHolders', offset: offset, limit: limit, searchPhrase: search_phrase};
                apiVms.post(data, onSuccessFunc, onFailureFunc);
            },
            function(msg){
                onFailureFunc(msg);
            },
            false
        );
    },

    getCardHolderPhoto: function (uid="", onSuccessFunc, onFailureFunc) {
        console.log('Sending getCardHolderPhoto request ... ');
        apiVms.checkToken(
            function(){
                let data = { token: cookieVms.get(), operation: 'getCardHolderPhoto', uid: uid};
                apiVms.post(data, onSuccessFunc, onFailureFunc);
            },
            function(msg){
                onFailureFunc(msg);
            },
            true
        );
    },

    emailCardHolder: function(uid, onSuccessFunc, onFailureFunc) {
        console.log('Sending emailCardHolder request ... ');
        const data = { token: cookieVms.get(), operation: 'emailCardHolder', uid: uid }
        apiVms.post(data, onSuccessFunc, onFailureFunc);
    },

    deleteCardHolder: function(uid, onSuccessFunc, onFailureFunc){
        console.log('Sending deleteCardHolder request ... ');
        apiVms.checkToken(
            function(){
                let data = { token: cookieVms.get(), operation: 'deleteCardHolder', uid: uid }
                apiVms.post(data, onSuccessFunc, onFailureFunc);
            },
            function(msg){
                onFailureFunc(msg);
            },
            false
        );
    },

    createCardHolder: function(firstName, lastName, company, email, fromDateTimeValid,
                                toDateTimeValid, photo, securityGroupUID, cardCode, onSuccessFunc, onFailureFunc) {
        console.log('Sending createCardHolder request ... ');
        apiVms.checkToken(
            function(){
                let data = {  token: cookieVms.get(),
                        operation: 'createCardHolder',
                        firstName: firstName,
                        lastName: lastName,
                        company: company,
                        email: email,
                        securityGroupUID: securityGroupUID,
                        cardCode: cardCode,
                        fromDateTimeValid: fromDateTimeValid,
                        toDateTimeValid: toDateTimeValid,
                        photo:photo };
                apiVms.post(data, onSuccessFunc, onFailureFunc);
            },
            function(msg){
                onFailureFunc(msg);
            },
            false
        );

    },

    getSecurityGroups: function (onSuccessFunc, onFailureFunc) {
        console.log('Sending getSecurityGroups request ... ');
        const data = { token: cookieVms.get(), operation: 'getSecurityGroups' };
        apiVms.post(data, onSuccessFunc, onFailureFunc);
    },

    post: function(data, onSuccessFunc, onFailureFunc) {
        let httpStatusCode = null;
        fetch(apiVms.url, {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify(data),
        })
        .then( response => {
            httpStatusCode = response.status;
            return response.text();
        })
        .then( bodyAsText => {
            try {
                return JSON.parse(bodyAsText);
            } catch (e) {
                if(httpStatusCode === 200){
                    return null;
                } else {
                    onFailureFunc("Request Failed with code (No JSON Body): " + httpStatusCode);
                }

            }
        })
        .then(bodyAsJson => {
            /*console.log('HTTP Status Code: ', httpStatusCode);
            console.log('bodyAsJson: ', bodyAsJson);*/
            if(bodyAsJson !== null) {
                if(bodyAsJson.hasOwnProperty('errorMessage')){
                    onFailureFunc(httpStatusCode + " - " + bodyAsJson.errorMessage);
                } else {
                    onSuccessFunc(bodyAsJson);
                }
            } else {
                onSuccessFunc(bodyAsJson);
            }

        })
        .catch((error) => {
          console.log('HTTP Status Code: ', httpStatusCode);
          console.error('Error:', error);
          if(Boolean(error.errorMessage)){
                console.error('Error message:', error.errorMessage);
          }
          onFailureFunc(error);
        });
    }
};