export var photo = {
    file_elem: null,
    preview_elem: null,
    spinner_elem: null,
    reader: null,
    unscaled_image: null,
    MAX_PHOTO_WIDTH: 350,
    MAX_PHOTO_HEIGHT: 450,
    custom_photo: false,


    setup: function(file_elem, preview_elem, spinner_elem){
        if(file_elem instanceof HTMLInputElement && file_elem.getAttribute('type') === 'file'){
            photo.file_elem = file_elem;
        } else {
            console.error("incorrect photo element type: " + input_elem.getAttribute('type'));
        }

        if(preview_elem instanceof HTMLImageElement){
            photo.preview_elem = preview_elem;
        } else {
            console.error("Photo preview element is not a IMG.");
        }

        if(spinner_elem instanceof HTMLDivElement){
            photo.spinner_elem = spinner_elem;
        } else {
            console.error("Spinner element is not a DIV.");
        }

        /* Logs when custom photo is successfully uploaded */
        photo.preview_elem.addEventListener("load", function () {
            console.log('photo.preview_elem size: ' + photo.preview_elem.width + 'x' + photo.preview_elem.height);
            photo.custom_photo = true;
        }, false);

        /* Runs a spinner only */
        photo.file_elem.onclick = function() {
            console.log("photo onclick");
            // Run the Spinner for 5 secs
            photo.spinner_elem.style.visibility = "visible";
            setTimeout(function(){ photo.spinner_elem.visibility = "hidden"; }, 5000);
        }

        /* Checks file is a Image and is not too big only */
        photo.file_elem.onchange  = function() { photo.onchange(); }

        /* Image is loaded into 'unscale_image' first, once loaded the image is resized and placed into preview_elem */
        photo.unscaled_image = new Image();
        photo.unscaled_image.addEventListener("load", function () { photo.resize(); }, false);

        photo.reader = new FileReader();
        photo.reader.addEventListener("load", function () {
            console.log("image loaded into unscaled_image");
            // convert image file to base64 string
            photo.unscaled_image.src = photo.reader.result;
            // Stop the Spinner
            photo.spinner_elem.style.visibility = "hidden";
        }, false);
    },

    /* Checks file is a Image and is not too big */
    onchange: function(){
        console.log("photo onchange");
        if(photo.file_elem.files[0]){
            if(!photo.file_elem.files[0].type.match(/image.*/)) {
                alert("File is not a image");
                return;
            }

            if(photo.file_elem.files[0].size > 10000000) {
                alert("Photo too big! (10mb Max)");
                photo.file_elem.value = "";
                photo.file_elem.files[0] = null;
                photo.preview_elem.src = "data:image/png;charset=utf-8;base64," + default_photo_src;
                photo.spinner_elem.style.visibility = "hidden";
                return;
            }

            /* Load the image into the reader */
            photo.reader.readAsDataURL(photo.file_elem.files[0]);

        } else {
            photo.spinner_elem.style.visibility = "hidden";
            photo.file_elem.files[0] = null;
        }
    },

    resize: function() {
        const canvas = document.createElement('canvas');
        console.log('photo.unscaled_image Image has changed');
        console.log('photo.unscaled_image size: ' + photo.unscaled_image.width + 'x' + photo.unscaled_image.height);
        if(photo.unscaled_image.width > photo.MAX_PHOTO_WIDTH) {
            let ratio = (photo.unscaled_image.width/photo.MAX_PHOTO_WIDTH);
            let new_height = photo.unscaled_image.height / ratio;
            let new_width = photo.unscaled_image.width / ratio;

            if(new_height > photo.MAX_PHOTO_HEIGHT) {
                ratio = (photo.unscaled_image.height/photo.MAX_PHOTO_HEIGHT);
                new_height = photo.unscaled_image.height / ratio;
                new_width = photo.unscaled_image.width / ratio;
            }

            canvas.width = new_width;
            canvas.height = new_height;
            canvas.getContext('2d').drawImage(photo.unscaled_image, 0, 0, photo.unscaled_image.width, photo.unscaled_image.height, 0, 0, new_width, new_height);
            photo.preview_elem.src = canvas.toDataURL('image/jpeg', 1.0);
        } else {
            photo.preview_elem.src = photo.unscaled_image.src;
        }
    }

};