
export var sigCanvas = {
    canvas: null,
    ctx: null,
    pos: { x: 0, y: 0 },
    offset: { left: 0, top: 0 },
    scroll: { x: 0, y: 0 },
    element: null,
    rect: null,

    init: function(element){
        this.canvas = document.createElement('canvas');
		this.canvas.setAttribute("id", "canvas_" + element.getAttribute('id'));
        this.element = element;
        this.ctx = this.canvas.getContext('2d');
        let rect = this.element.getBoundingClientRect();
        this.scroll.x = window.scrollX;
        this.scroll.y = window.scrollY;
        this.offset.left = rect.left;
        this.offset.top = rect.top;
        this.canvas.width = rect.width;
        this.canvas.height = rect.height;
        this.element.appendChild(this.canvas);
    },

    refresh: function(){
        let rect = this.element.getBoundingClientRect();
        this.scroll.x = window.scrollX;
        this.scroll.y = window.scrollY;
        this.offset.left = rect.left;
        this.offset.top = rect.top;
    },

    clear: function(){
        this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
    },

    getSnapShot: function(e) {
        if (!this.canvas) {
            console.error("this must be initialised with init");
            return
        } else {
            return this.canvas.toDataURL();
        }
    },

    setTouchPos: function(e) {
        //console.log("setTouchPos");
        if (!e)
            var e = event;

        if(e.touches) {
            if (e.touches.length == 1) { // Only deal with one finger
                var touch = e.touches[0]; // Get the information for finger #1
                var rect = e.target.getBoundingClientRect();
                this.offset.left = rect.left;
                this.offset.top = rect.top;
                this.scroll.x = window.scrollX;
                this.scroll.y = window.scrollY;
                /*this.pos.x=touch.pageX-touch.target.offsetLeft;
                this.pos.y=touch.pageY-touch.target.offsetTop;*/
                this.pos.x = e.targetTouches[0].pageX - (this.offset.left + this.scroll.x);
                this.pos.y = e.targetTouches[0].pageY - (this.offset.top + this.scroll.y);
            }
        }
        e.preventDefault();
        //alert("this.pos.y:" + this.pos.y);
    },

    setMousePos: function(e) {
        console.log("setMousePos");
        //if((this.scroll.x != window.scrollX) || (this.scroll.y != window.scrollY)){
            this.scroll.x = window.scrollX;
            this.scroll.y = window.scrollY;
            let rect = this.element.getBoundingClientRect();
            this.offset.left = rect.left;
            this.offset.top = rect.top;
        //} else {
        //    console.log("No Scroll");
        //}
        this.pos.x = e.pageX - (this.offset.left + this.scroll.x);
        this.pos.y = e.pageY - (this.offset.top + this.scroll.y);
    },

    mouseDraw: function(e) {
        if (!this.ctx) { console.error("this must be initialised with init"); return}
        // mouse left button must be pressed
        if (e.buttons !== 1) return;

        this.ctx.beginPath(); // begin

        this.ctx.lineWidth = 5;
        this.ctx.lineCap = 'round';
        this.ctx.strokeStyle = '#c0392b';

        this.ctx.moveTo(this.pos.x, this.pos.y); // from
        //console.log("Drawing from x:" + this.pos.x + " y:" + this.pos.y);
        this.setMousePos(e);
        this.ctx.lineTo(this.pos.x, this.pos.y); // to
        //console.log("Drawing to x:" + this.pos.x + " y:" + this.pos.y);


        this.ctx.stroke(); // draw it!

        e.preventDefault();
    },

    touchDraw: function(e) {
        if (!this.ctx) { console.error("this must be initialised with init"); return}
        // Update the touch co-ordinates
        //this.setTouchPos(e);

        this.ctx.beginPath(); // begin

        this.ctx.lineWidth = 5;
        this.ctx.lineCap = 'round';
        this.ctx.strokeStyle = '#c0392b';

        this.ctx.moveTo(this.pos.x, this.pos.y); // from
        this.setTouchPos(e);
        this.ctx.lineTo(this.pos.x, this.pos.y); // to

        this.ctx.stroke(); // draw it!

        // Prevent a scrolling action as a result of this touchmove triggering.
        e.preventDefault();
    }
};