import {apiVms} from "/js/modules/api_vms.mjs";
import {cookieVms} from "/js/modules/cookie.mjs";

const CARDHOLDER_OVERFLOW = 5; // Extra non visible cardholders

function poll(fn, interval)
{
    fn();
    if(POLLING)setTimeout(poll, interval, fn, interval);
    return;
};

function refreshTable(site='onsite'){
    let status = document.getElementById("conn_status");
    status.textContent = 'Connecting ...';
    let total = document.getElementById("onsite_total");
    let scroll = document.getElementById("onsite_list_scroll");
    let list = document.getElementById("onsite_list_tbl");
    let getCountFunc = apiVms.getOnsiteCount;
    let getCardholdersFunc = apiVms.getOnsiteCardholders;
    if(site != 'onsite'){
        total = document.getElementById("offsite_total");
        scroll = document.getElementById("offsite_list_scroll");
        list = document.getElementById("offsite_list_tbl");
        getCountFunc = apiVms.getOffsiteCount;
        getCardholdersFunc = apiVms.getOffsiteCardholders;
    }

    getCountFunc(
        function(data) {
            // Update total
            total.textContent = data.count;
            // Update Num of Rows
            let row_count = list.rows.length;
            while(row_count > data.count){
                list.deleteRow(row_count-1);
                row_count--;
            }
            while(row_count < data.count){
                let row = list.insertRow();
                row.innerHTML = "<span style='opacity: 0.3;'>&#8635;" + row_count + "<span>";
                row_count++;
            }

            // Workout top row visible
            let i = 0;
            let num_rows = 0;
            if (list.rows.length > 1){
                // Calc Top Row
                if(scroll.scrollTop > list.rows[0].offsetHeight){
                    i = Math.floor(scroll.scrollTop / list.rows[0].offsetHeight);
                }

                // Calc Num of Rows
                let computedStyle = getComputedStyle(list);
                let unpadded_list_height = scroll.clientHeight - (parseFloat(computedStyle.paddingTop) + parseFloat(computedStyle.paddingBottom));  // height without padding
                num_rows = Math.floor((unpadded_list_height / list.rows[0].offsetHeight) + CARDHOLDER_OVERFLOW);
            }

            console.log("Getting Rows " + i + " to " + (i + num_rows));
            getCardholdersFunc(
                i, num_rows,
                function(data) {
                    status.textContent = 'Online';
                    for (let key in data) {
                        if (data.hasOwnProperty(key)) {
                            populate_row(list.rows[i], data[key]);
                            i++;
                        }
                    }
                },
                function(msg) {
                    status.textContent = 'Offline';
                    console.log("Fail: " + msg);
                }
            )
        },
        function(msg)
        {
            status.textContent = 'Offline';
            console.log("Fail: " + msg);
        }
    )
    return true;
}

function populate_row(row, cardholder){

  let photo_src = "/images/default_photo.png";
  if(!cardholder.lastPassDate){
      cardholder.lastPassDate = "";
  }

  // Override with existing Photo to mitigate against flicker
  if(row.getElementsByClassName('cardholder_uid').length > 0){
    let current_cardholder_uid = row.getElementsByClassName('cardholder_uid')[0].value;
    let current_lastPass_value = row.getElementsByClassName('lastPass_value')[0].innerText;
    let current_area_value = row.getElementsByClassName('area_value')[0].innerText;
    if((cardholder.uid == current_cardholder_uid) &&
       (cardholder.lastPassDate == current_lastPass_value) &&
        (cardholder.insideArea.name == current_area_value)) {
        return;
    }
  }
  if(row.getElementsByClassName('cardholder_photo').length > 0){
    photo_src = row.getElementsByClassName('cardholder_photo')[0].src;
  }

  row.innerHTML = "";
  let record = document.createElement("div");
  record.classList.add('cardholder_record');

  let photo = document.createElement("img");
  photo.classList.add('cardholder_photo');
  photo.src = photo_src;
  record.appendChild(photo);

  let name = document.createElement("div");
  name.classList.add('cardholder_name');
  name.textContent = cardholder.firstName  + ' ' +  cardholder.lastName;
  record.appendChild(name);

  let area = document.createElement("div");
  area.classList.add('cardholder_area');

      let area_label = document.createElement("span");
      area_label.classList.add('area_label');
      area_label.textContent = 'Area';
      area.appendChild(area_label);

      let area_value = document.createElement("span");
      area_value.classList.add('area_value');
      area_value.textContent = cardholder.insideArea.name;
      area.appendChild(area_value);

  record.appendChild(area);

  let lastPass = document.createElement("div");
  lastPass.classList.add('cardholder_lastPass');

      let lastPass_label = document.createElement("span");
      lastPass_label.classList.add('lastPass_label');
      lastPass_label.textContent = 'Last Pass';
      lastPass.appendChild(lastPass_label);

      let lastPass_value = document.createElement("span");
      lastPass_value.classList.add('lastPass_value');
      lastPass_value.innerText = cardholder.lastPassDate;
      lastPass.appendChild(lastPass_value);

  record.appendChild(lastPass);

  let cardholder_uid = document.createElement("input");
  cardholder_uid.classList.add('cardholder_uid');
  cardholder_uid.style.visibility = "hidden";
  cardholder_uid.value = cardholder.uid;
  record.appendChild(cardholder_uid);

  row.appendChild(record);

  apiVms.getCardHolderPhoto(
            cardholder.uid,
            function(data) {
                if(data.photo){
                    photo.src = 'data:image/png;base64, ' + data.photo;
                } else {
                    photo.src = "/images/default_photo.png";
                }
            },
            function(msg) {
                console.log("Failed to load new photo: " + msg);
            }
        )
}

var POLLING = false;
const POLL_INTERVAL = 10000;

function autoPollBtnClick(element) {
    if(POLLING) {
        POLLING = false;
        element.textContent = "Start Auto Refresh";
        return;
    }
    if(!POLLING) {
        POLLING = true;
        poll(window.refreshBtnClick,POLL_INTERVAL);
        element.textContent = "Stop Auto Refresh";
        return;
    }
}

/* Expose function to global scope */
window.autoPollBtnClick = autoPollBtnClick;
window.refreshBtnClick = () => { refreshTable('onsite'); refreshTable('offsite'); }

var token = cookieVms.get()

// Initial Page load
window.onload = function(){
    if(token == null) {
        console.log("no token present");
        apiVms.getToken("guest", "visitor",
            function(data) {  /* OnSuccess */
                cookieVms.set(data.token);
            },
            function(msg) {  /* OnFailure */
                console.log("Failed to get token: " + msg);
            })
    }

    /* First population of the two lists */
    //refreshTable('onsite');
    //refreshTable('offsite');
    /* Start Auto Polling */
    autoPollBtnClick(document.getElementById("autoPollBtn"));

    /* Listen to scroll events on the two lists */
    let offsite_scroll = document.getElementById("offsite_list_scroll");
    var offsite_scroll_timer = null;
    offsite_scroll.addEventListener('scroll', function() {
        if(offsite_scroll_timer !== null) {
            clearTimeout(offsite_scroll_timer);
        }
        offsite_scroll_timer = setTimeout(function() {
              refreshTable('offsite');
        }, 150);
    }, false);

    let onsite_scroll = document.getElementById("onsite_list_scroll");
    var onsite_scroll_timer = null;
    onsite_scroll.addEventListener('scroll', function() {
        if(onsite_scroll_timer !== null) {
            clearTimeout(onsite_scroll_timer);
        }
        onsite_scroll_timer = setTimeout(function() {
              refreshTable('onsite');
        }, 150);
    }, false);

};
