import {apiVms} from "/js/modules/api_vms.mjs";
import {cookieVms} from "/js/modules/cookie.mjs";
import {sigCanvas} from "/js/modules/signature_canvas.mjs";

const sigCanvas1 =  Object.create(sigCanvas);
const sigCanvas2 =  Object.create(sigCanvas);

/* Fake Delay to provide time to view Spinner */
function fake_net_delay() {
   var currentTime = new Date().getTime();
   while (currentTime + 5000 >= new Date().getTime()) {
   }
}

function onBeginBtnClick(){
    let splash_div = document.getElementById("splash");
    let start_over_btn = document.getElementById("start_over_btn");
    let search_form = document.getElementById("visitor_search_form");

    splash_div.remove();
    search_form.style.display = "block";
    start_over_btn.style.display = "block";
}

function emailSearchBtnClick() 
{
    let email                = document.getElementById("visitorEmail").value;
    let spinner              = document.getElementById("emailSearchSpinner");

    /* Show the spinner */
    spinner.style.display = "block";

    /* Change Mouse pointer to a hand */
    document.getElementById("visitor_found_form").style.cursor = "pointer";

    apiVms.emailSearchCardHolder(email,
        function (cardholder) { /* OnSuccess */
            //fake_net_delay();
            /* Remove the Visitor Search Form */
            const element = document.getElementById("visitor_search_form");
            element.remove();

			/* Populate & Display Visitor Found Table */
			let dateFrom = new Date(cardholder.fromDateValid);
			let dateTo = new Date(cardholder.toDateValid);

			let uid = document.getElementById("visitor_uid");
			uid.value = cardholder.uid;

			let visitor_found_form   = document.getElementById("visitor_found_form");
            visitor_found_form.innerHTML = visitor_found_form.innerHTML
                            .replace('%firstName%',cardholder.firstName)
                            .replace('%lastName%',cardholder.lastName)
                            .replace('%company%',cardholder.cardholderPersonalDetail.company)
						    .replace('%fromDateValid%',dateFrom.toLocaleString())
							.replace('%toDateValid%',dateTo.toLocaleString());
			visitor_found_form.style.display = "block";
            },
        function (msg) { /* OnFailure */
            //fake_net_delay();
            console.log("Cardholder search failed - " + msg);
            /* Remove the Visitor Search Form */
            const element = document.getElementById("visitor_search_form");
            element.remove();
            /* Display Failure Message */
            let submission_failure_div   = document.getElementById("submission_failure");
            submission_failure.innerHTML  = "<p>Cardholder search failed - " + msg + "</p>";
            submission_failure.style.display = "block";
           },
    );


	return false;
}

function cardholderSelectedBtnClick(){
	/* Remove Found Visitor Table */
    const element = document.getElementById("visitor_found_form");
    element.remove();
    /* Display Signatures and Work Policy */
	let signatures_form    = document.getElementById("signatures");
    signatures_form.style.display = "block"

    // create canvas element and append it to document body
	let sig_box = document.getElementById("signature_box");
    sigCanvas1.init(sig_box);

    let sig_box2 = document.getElementById("signature_boxHost");
    sigCanvas2.init(sig_box2);

    sig_box.addEventListener('mousemove', function mousemove1(e){ sigCanvas1.mouseDraw(e); } );
    sig_box.addEventListener('mousedown', function mousedown1(e){ sigCanvas1.setMousePos(e); });
    sig_box.addEventListener('mouseenter', function mouseenter1(e){ sigCanvas1.setMousePos(e); });
    sig_box.addEventListener('touchmove', function touchmove1(e){ sigCanvas1.touchDraw(e); });
    sig_box.addEventListener('touchstart', function touchstart1(e){ sigCanvas1.setTouchPos(e); });

	sig_box2.addEventListener('mousemove', function mousemove2(e){ sigCanvas2.mouseDraw(e); });
    sig_box2.addEventListener('mousedown', function mousedown2(e){ sigCanvas2.setMousePos(e); });
    sig_box2.addEventListener('mouseenter', function mouseenter2(e){ sigCanvas2.setMousePos(e); });
    sig_box2.addEventListener('touchmove', function touchmove2(e){ sigCanvas2.touchDraw(e); });
    sig_box2.addEventListener('touchstart', function touchstart2(e){ sigCanvas2.setTouchPos(e);});

	console.log("button pressed");
}

function signatureSubmitBtnClick(){
    let uid                = document.getElementById("visitor_uid").value;
    let v_sig              = sigCanvas1.getSnapShot();
    let h_sig              = sigCanvas2.getSnapShot();
    let nda                = true;

    if (v_sig.match(/^data:image\/(png|jpg|jpeg);base64,/,)) {
        v_sig = v_sig.replace(/^data:image\/(png|jpg|jpeg);base64,/, "");
    }

    if (h_sig.match(/^data:image\/(png|jpg|jpeg);base64,/,)) {
        h_sig = h_sig.replace(/^data:image\/(png|jpg|jpeg);base64,/, "");
    }

    apiVms.updateCardHolderSignatures(uid, v_sig, h_sig, nda,
        function () { /* OnSuccess */
                console.log("Signature Submission Successful");
                /* Hide Signature Form */
               let signatures_form    = document.getElementById("signatures");
               signatures_form.style.display = "none"
                /* Show Success Div */
                let submission_failure_div   = document.getElementById("signature_submission_success");
                signature_submission_success.style.display = "block";
            },
        function (msg) { /* OnFailure */
               console.log("Signature Submission Failure: " + msg);
               /* Hide Signature Form */
               let signatures_form    = document.getElementById("signatures");
               signatures_form.style.display = "none"
               /* Show Failure Message */
               let submission_failure_div   = document.getElementById("submission_failure");
               submission_failure.innerHTML  = "<p>Signature submission failed - " + msg + "</p>";
               submission_failure.style.display = "block";
           },
        );
    return true;
}

function clearClientSigCanvasBtnClick(){
    sigCanvas1.clear();
    return false;
}
function clearHostSigCanvasBtnClick(){
    sigCanvas2.clear();
    return false;
}

/* Expose function to global scope */
window.emailSearchBtnClick = emailSearchBtnClick;
window.cardholderSelectedBtnClick = cardholderSelectedBtnClick;
window.clearClientSigCanvasBtnClick = clearClientSigCanvasBtnClick;
window.clearHostSigCanvasBtnClick = clearHostSigCanvasBtnClick;
window.signatureSubmitBtnClick = signatureSubmitBtnClick;
window.onBeginBtnClick = onBeginBtnClick;

var token = cookieVms.get()

// Initial Page load
window.onload = function(){
    if(token == null) {
        console.log("no token present")
        apiVms.getToken("guest", "visitor",
            function(data) {  /* OnSuccess */
                cookieVms.set(data.token);
            },
            function(msg) {  /* OnFailure */
                /* Hide Splash */
                let splash_div = document.getElementById("splash");
                let start_over_btn = document.getElementById("start_over_btn");
                splash_div.remove();
                start_over_btn.style.display = "block";
                /* Show Failure Message */
                let submission_failure_div   = document.getElementById("submission_failure");
                submission_failure.innerHTML  = "<p>Connection failed - " + msg + "</p>";
                submission_failure.style.display = "block";
            })
    }
};

